@@ -28,6 +28,10 @@ Set to a true value if dpkg-deb is available.
 After the build stage, set to a hash reference of the directories we moved
 files from and to, so these moves can be reverted in the cleantree stage.
 
+=item fixperms
+
+If this is set to true, the generated debian/rules will run dh_fixperms.
+
 =back
 
 =head1 METHODS
@@ -41,17 +45,23 @@ if it is available since it is a lot more future-proof.
 
 =cut
 
+sub _inpath {
+	my $this=shift;
+	my $program=shift;
+
+	foreach (split(/:/,$ENV{PATH})) {
+		if (-x "$_/$program") {
+			return 1;
+		}
+	}
+	return '';
+}
+
 sub init {
 	my $this=shift;
 	$this->SUPER::init(@_);
 
-	$this->have_dpkg_deb('');
-	foreach (split(/:/,$ENV{PATH})) {
-		if (-x "$_/dpkg-deb") {
-			$this->have_dpkg_deb(1);
-			last;
-		}
-	}
+	$this->have_dpkg_deb($this->_inpath('dpkg-deb'));
 }
 
 =item checkfile
@@ -77,10 +87,33 @@ sub install {
 	my $this=shift;
 	my $deb=shift;
 
-	system("dpkg", "--no-force-overwrite", "-i", $deb) == 0
+	$this->do("dpkg", "--no-force-overwrite", "-i", $deb)
 		or die "Unable to install";
 }
 
+=item test
+
+Test a deb with lintian. Pass in the filename of the deb to test.
+
+=cut
+
+sub test {
+	my $this=shift;
+	my $deb=shift;
+
+	if ($this->_inpath("lintian")) {
+		# Ignore some lintian warnings that don't matter for
+		# aliened packages.
+		return map { s/\n//; $_ }
+		       grep {
+		       		! /unknown-section alien/
+		       } $this->runpipe("lintian $deb");
+	}
+	else {
+		return "lintian not available, so not testing";
+	}
+}
+
 =item getcontrolfile
 
 Helper method. Pass it the name of a control file, and it will pull it out
@@ -94,7 +127,7 @@ sub getcontrolfile {
 	my $file=$this->filename;
 	
 	if ($this->have_dpkg_deb) {
-		return `dpkg-deb --info $file $controlfile 2>/dev/null`;
+		return $this->runpipe("dpkg-deb --info $file $controlfile 2>/dev/null");
 	}
 	else {
 		# Solaris tar doesn't support O
@@ -103,13 +136,11 @@ sub getcontrolfile {
 
 			return "(mkdir /tmp/tar_out.$$ &&".
 				" cd /tmp/tar_out.$$ &&".
-				# Have to handle old debs without a leading ./ and
-				# new ones with it.
-				" tar xf - $file ./$file &&".
+				" tar xf - ./$file &&".
 				" cat $file; cd /; rm -rf /tmp/tar_out.$$)";
 		}
 		my $getcontrol = "ar -p $file control.tar.gz | gzip -dc | ".tar_out($controlfile)." 2>/dev/null";
-		return `$getcontrol`
+		return $this->runpipe($getcontrol);
 	}
 }
 
@@ -174,15 +205,15 @@ sub scan {
 	$this->conffiles(\@conffiles);
 
 	# Read in the list of all files.
-	# Note that tar doesn't supply a leading `/', so we have to add that.
+	# Note that tar doesn't supply a leading '/', so we have to add that.
 	my @filelist;
 	if ($this->have_dpkg_deb) {
 		@filelist=map { chomp; s:\./::; "/$_" }
-			  `dpkg-deb --fsys-tarfile $file | tar tf -`;
+			  $this->runpipe("dpkg-deb --fsys-tarfile $file | tar tf -");
 	}
 	else {
 		@filelist=map { chomp; s:\./::; "/$_" }
-			  `ar -p $file data.tar.gz | gzip -dc | tar tf -`;
+			  $this->runpipe("ar -p $file data.tar.gz | gzip -dc | tar tf -");
 	}
 	$this->filelist(\@filelist);
 
@@ -206,12 +237,12 @@ sub unpack {
 	my $file=$this->filename;
 
 	if ($this->have_dpkg_deb) {
-		system("dpkg-deb", "-x", $file, $this->unpacked_tree) == 0
-			or die "Unpacking of `$file' failed: $!";
+		$this->do("dpkg-deb", "-x", $file, $this->unpacked_tree)
+			or die "Unpacking of '$file' failed: $!";
 	}
 	else {
-		system("ar -p $file data.tar.gz | gzip -dc | (cd ".$this->unpacked_tree."; tar xpf -)") == 0
-			or die "Unpacking of `$file' failed: $!";
+		$this->do("ar -p $file data.tar.gz | gzip -dc | (cd ".$this->unpacked_tree."; tar xpf -)")
+			or die "Unpacking of '$file' failed: $!";
 	}
 
 	return 1;
@@ -220,24 +251,25 @@ sub unpack {
 =item getpatch
 
 This method tries to find a patch file to use in the prep stage. If it
-finds one, it returns it.  Pass in a list of directories to search for
+finds one, it returns it. Pass in a list of directories to search for
 patches in.
 
 =cut
 
 sub getpatch {
 	my $this=shift;
-
+	my $anypatch=shift;
+	
 	my @patches;
 	foreach my $dir (@_) {
 		push @patches, glob("$dir/".$this->name."_".$this->version."-".$this->release."*.diff.gz");
 	}
-	unless (@patches) {
-		# Try not matching the revision, see if that helps.
+	if (! @patches) {
+		# Try not matching the release, see if that helps.
 		foreach my $dir (@_) {
 			push @patches,glob("$dir/".$this->name."_".$this->version."*.diff.gz");
 		}
-		unless (@patches) {
+		if (@patches && $anypatch) {
 			# Fallback to anything that matches the name.
 			foreach my $dir (@_) {
 				push @patches,glob("$dir/".$this->name."_*.diff.gz");
@@ -261,20 +293,35 @@ sub prep {
 	my $this=shift;
 	my $dir=$this->unpacked_tree || die "The package must be unpacked first!";
 
-	mkdir "$dir/debian", 0755 ||
+	$this->do("mkdir $dir/debian") ||
 		die "mkdir $dir/debian failed: $!";
 	
 	# Use a patch file to debianize?
 	if (defined $this->patchfile) {
 		# The -f passed to zcat makes it pass uncompressed files
 		# through without error.
-		system("zcat -f ".$this->patchfile." | (cd $dir; patch -p1)") == 0
+		$this->do("zcat -f ".$this->patchfile." | (cd $dir; patch -p1)")
 			or die "patch error: $!";
 		# Look for .rej files.
 		die "patch failed with .rej files; giving up"
-			if `find $dir -name "*.rej"`;
-		system('find', '.', '-name', '*.orig', '-exec', 'rm', '{}', ';');
-		chmod 0755, "$dir/debian/rules";
+			if $this->runpipe("find $dir -name \"*.rej\"");
+		$this->do('find', '.', '-name', '*.orig', '-exec', 'rm', '{}', ';');
+		$this->do("chmod", 755, "$dir/debian/rules");
+
+		# It's possible that the patch file changes the debian
+		# release or version. Parse changelog to detect that.
+		open (my $changelog, "<$dir/debian/changelog") || return;
+		my $line=<$changelog>;
+		if ($line=~/^[^ ]+\s+\(([^)]+)\)\s/) {
+			my $version=$1;
+			if ($version=~/(.*)-(.*)/) {
+				$version=$1;
+				$this->release($2);
+			}
+			$this->version($1);
+		}
+		close $changelog;
+		
 		return;
 	}
 
@@ -284,6 +331,7 @@ sub prep {
 	print OUT $this->name." (".$this->version."-".$this->release.") experimental; urgency=low\n";
 	print OUT "\n";
 	print OUT "  * Converted from .".$this->origformat." format to .deb\n";
+	print OUT "    by alien version $Alien::Version\n";
 	print OUT "\n";
 	print OUT " -- ".$this->username." <".$this->email.">  ".$this->date."\n";
 	print OUT "\n";
@@ -320,8 +368,8 @@ sub prep {
 	print OUT $this->binary_info."\n";
 	close OUT;
 
-	# Conffiles, if any.
-	my @conffiles=@{$this->conffiles};
+	# Conffiles, if any. Note that debhelper takes care of files in /etc.
+	my @conffiles=grep { $_ !~ /^\/etc/ } @{$this->conffiles};
 	if (@conffiles) {
 		open (OUT, ">$dir/debian/conffiles") || die "$dir/debian/conffiles: $!";
 		print OUT join("\n", @conffiles)."\n";
@@ -330,7 +378,8 @@ sub prep {
 
 	# A minimal rules file.
 	open (OUT, ">$dir/debian/rules") || die "$dir/debian/rules: $!";
-	print OUT << 'EOF';
+	my $fixpermscomment = $this->fixperms ? "" : "#";
+	print OUT << "EOF";
 #!/usr/bin/make -f
 # debian/rules for alien
 
@@ -340,8 +389,7 @@ sub prep {
 # Use v3 compatability mode, so ldconfig gets added to maint scripts.
 export DH_COMPAT=3
 
-PACKAGE=$(shell dh_listpackages)
-PKGFILES=$(shell ls -1 |grep -v debian)
+PACKAGE=\$(shell dh_listpackages)
 
 build:
 	dh_testdir
@@ -359,22 +407,23 @@ binary-arch: build
 	dh_clean -k
 	dh_installdirs
 
-# Copy the packages's files, if any.
-ifneq "$(PKGFILES)" ""
-	cp -a $(PKGFILES) debian/$(PACKAGE)
-endif
-
-#
-# If you need to move files around in debian/$(PACKAGE) or do some
-# binary patching, do it here
-#
 	dh_installdocs
 	dh_installchangelogs
+
+# Copy the packages's files.
+	find . -maxdepth 1 -mindepth 1 -not -name debian -print0 | \\
+		xargs -0 -r -i cp -a {} debian/\$(PACKAGE)
+
+#
+# If you need to move files around in debian/\$(PACKAGE) or do some
+# binary patching, do it here
+#
+
+
 # This has been known to break on some wacky binaries.
 #	dh_strip
 	dh_compress
-# This is too paranoid to be generally useful to alien.
-#	dh_fixperms
+$fixpermscomment	dh_fixperms
 	dh_makeshlibs
 	dh_installdeb
 	-dh_shlibdeps
@@ -386,7 +435,7 @@ binary: binary-indep binary-arch
 .PHONY: build clean binary-indep binary-arch binary
 EOF
 	close OUT;
-	chmod 0755,"$dir/debian/rules";
+	$this->do("chmod", 755, "$dir/debian/rules");
 
 	# Save any scripts.
 	if ($this->usescripts) {
@@ -411,10 +460,10 @@ EOF
 		if (-d "$dir/$olddir" && ! -e "$dir/$dirtrans{$olddir}") {
 			# Ignore failure..
 			my ($dirbase)=$dirtrans{$olddir}=~/(.*)\//;
-			system("install", "-d", "$dir/$dirbase");
-			system("mv", "$dir/$olddir", "$dir/$dirtrans{$olddir}");
+			$this->do("install", "-d", "$dir/$dirbase");
+			$this->do("mv", "$dir/$olddir", "$dir/$dirtrans{$olddir}");
 			if (-d "$dir/$olddir") {
-				system("rmdir", "-p", "$dir/$olddir");
+				$this->do("rmdir", "-p", "$dir/$olddir");
 			}
 		}
 		else {
@@ -434,7 +483,7 @@ sub build {
 	my $this=shift;
 
 	chdir $this->unpacked_tree;
-	my $log=`debian/rules binary 2>&1`;
+	my $log=$this->runpipe("debian/rules binary 2>&1");
 	if ($?) {
 		die "Package build failed. Here's the log:\n", $log;
 	}
@@ -458,15 +507,15 @@ sub cleantree {
 		if (! -e "$dir/$olddir" && -d "$dir/$dirtrans{$olddir}") {
 			# Ignore failure.. (should I?)
 			my ($dirbase)=$dir=~/(.*)\//;
-			system("install", "-d", "$dir/$dirbase");
-			system("mv", "$dir/$dirtrans{$olddir}", "$dir/$olddir");
+			$this->do("install", "-d", "$dir/$dirbase");
+			$this->do("mv", "$dir/$dirtrans{$olddir}", "$dir/$olddir");
 			if (-d "$dir/$dirtrans{$olddir}") {
-				system("rmdir", "-p", "$dir/$dirtrans{$olddir}");
+				$this->do("rmdir", "-p", "$dir/$dirtrans{$olddir}");
 			}
 		}
 	}
 	
-	system("rm", "-rf", "$dir/debian");
+	$this->do("rm", "-rf", "$dir/debian");
 }
 
 =item package
@@ -585,7 +634,7 @@ sub description {
 	}
 	$ret=~s/^\n+//g; # kill leading blank lines
 	$ret.=" .\n" if length $ret;
-	$ret.=" (Converted from a ".$this->origformat." package by alien.)";
+	$ret.=" (Converted from a ".$this->origformat." package by alien version $Alien::Version.)";
 	return $ret;
 }
 
@@ -598,7 +647,7 @@ Returns the date, in rfc822 format.
 sub date {
 	my $this=shift;
 
-	my $date=`822-date`;
+	my $date=$this->runpipe("822-date");
 	chomp $date;
 	if (!$date) {
 		die "822-date did not return a valid result. You probably need to install the dpkg-dev debian package";
@@ -628,7 +677,7 @@ sub email {
 		close MAILNAME;
 	}
 	if (!$mailname) {
-		$mailname=`hostname -f`;
+		$mailname=$this->runpipe("hostname -f");
 		chomp $mailname;
 	}
 	return "$login\@$mailname";
@@ -658,6 +707,45 @@ sub username {
 	return $username;
 }
 
+=item postinst
+
+Returns the postinst. This may include generated shell code to set owners
+and groups from the owninfo field, and update modes from the modeinfo field.
+
+=cut
+
+sub postinst {
+	my $this=shift;
+
+	my $owninfo = $this->owninfo;
+	my $modeinfo = $this->modeinfo;
+	my $postinst = $this->{postinst};
+	return $postinst unless ref $owninfo;
+
+	# If there is no postinst, let's make one up..
+	$postinst="#!/bin/sh\n" unless length $postinst;
+	
+	return $postinst unless %$owninfo;
+	
+	my ($firstline, $rest)=split(/\n/, $postinst, 2);
+	if ($firstline !~ m/^#!\s*\/bin\/sh/) {
+		print STDERR "warning: unable to add ownership fixup code to postinst as the postinst is not a shell script!\n";
+		return $postinst;
+	}
+
+	my $permscript="# alien added permissions fixup code\n";
+	foreach my $file (sort keys %$owninfo) {
+		my $quotedfile=$file;
+		$quotedfile=~s/'/'"'"'/g; # no single quotes in single quotes..
+		$permscript.="chown '".$owninfo->{$file}."' '$quotedfile'\n";
+		$permscript.="chmod '".$modeinfo->{$file}."' '$quotedfile'\n"
+			if (defined $modeinfo->{$file});
+	}
+	return "$firstline\n$permscript\n$rest";
+}
+
+=cut
+
 =head1 AUTHOR
 
 Joey Hess <joey@kitenet.net>
