@@ -23,6 +23,8 @@ Alien::Package.
 
 Relocatable rpm packages have a prefixes field.
 
+=back
+
 =head1 METHODS
 
 =over 4
@@ -51,7 +53,7 @@ sub install {
 	my $this=shift;
 	my $rpm=shift;
 
-	system("rpm -ivh ".(exists $ENV{RPMINSTALLOPT} ? $ENV{RPMINSTALLOPT} : '').$rpm) == 0
+	$this->do("rpm -ivh ".(exists $ENV{RPMINSTALLOPT} ? $ENV{RPMINSTALLOPT} : '').$rpm)
 		or die "Unable to install";
 }
 
@@ -81,23 +83,23 @@ sub scan {
 
 	# Use --queryformat to pull out all the fields we need.
 	foreach my $field (keys(%fieldtrans)) {
-		$_=`LANG=C rpm -qp --queryformat \%{$field} $file`;
+		$_=$this->runpipe("LANG=C rpm -qp --queryformat \%{$field} $file");
 		$field=$fieldtrans{$field};
 		$_='' if $_ eq '(none)';
 		$this->$field($_);
 	}
 
 	# Get the conffiles list.
-	$this->conffiles([map { chomp; $_ } `LANG=C rpm -qcp $file`]);
+	$this->conffiles([map { chomp; $_ } $this->runpipe("LANG=C rpm -qcp $file")]);
 	if (defined $this->conffiles->[0] &&
 	    $this->conffiles->[0] eq '(contains no files)') {
 		$this->conffiles([]);
 	}
 
-	$this->binary_info(scalar `rpm -qpi $file`);
+	$this->binary_info(scalar $this->runpipe("rpm -qpi $file"));
 
 	# Get the filelist.
-	$this->filelist([map { chomp; $_ } `LANG=C rpm -qpl $file`]);
+	$this->filelist([map { chomp; $_ } $this->runpipe("LANG=C rpm -qpl $file")]);
 	if (defined $this->filelist->[0] &&
 	    $this->filelist->[0] eq '(contains no files)') {
 		$this->filelist([]);
@@ -145,8 +147,8 @@ sub unpack {
 	$this->SUPER::unpack(@_);
 	my $workdir=$this->unpacked_tree;
 	
-	system("rpm2cpio ".$this->filename." | (cd $workdir; cpio --extract --make-directories --no-absolute-filenames --preserve-modification-time) 2>/dev/null") == 0
-		or die "Unpacking of `".$this->filename."' failed";
+	$this->do("rpm2cpio ".$this->filename." | (cd $workdir; cpio --extract --make-directories --no-absolute-filenames --preserve-modification-time) 2>&1")
+		or die "Unpacking of '".$this->filename."' failed";
 	
 	# If the package is relocatable. We'd like to move it to be under
 	# the $this->prefixes directory. However, it's possible that that
@@ -168,46 +170,65 @@ sub unpack {
 		foreach (split m:/:, $this->prefixes) {
 			if ($_ ne '') { # this keeps us from using anything but relative paths.
 				$collect.="/$_";
-				mkdir $collect,0755 || die "unable to mkdir $collect: $!";
+				$this->do("mkdir", $collect) || die "unable to mkdir $collect: $!";
 			}
 		}
 		# Now move all files in the package to the directory we made.
-		system("mv", @filelist, "$workdir/".$this->prefixes) == 0
-			or die "error moving unpacked files into the default prefix directory: $!";
+		if (@filelist) {
+			$this->do("mv", @filelist, "$workdir/".$this->prefixes)
+				or die "error moving unpacked files into the default prefix directory: $!";
+		}
 	}
 
-	# When cpio extracts the file, any child directories that are
-	# present, but whose parent directories are not, end up mode 700.
-	# This next block corrects that to 755, which is more reasonable.
-	#
-	# Of course, this whole thing assumes we get the filelist in sorted
-	# order.
-	my $lastdir='';
-	my %tochmod;
-	foreach my $file (@{$this->filelist}) {
-		$file=~s/^\///;
-		if (($lastdir && $file !~ m:^\Q$lastdir\E/[^/]*$:) || !$lastdir) {
-			# We've found one of the nasty directories. Fix it
-			# up.
-			#
-			# Note that I strip the trailing filename off $file
-			# here, for two reasons. First, it makes the loop
-			# easier, we don't need to fix the perms on the
-			# last file, after all! Second, it makes the -d
-			# test below fire, which saves us from trying to
-			# fix a parent directory twice.
-			$file=$1 if $file=~m:(.*)/.*?:;
-			my $dircollect='';
-			
-			foreach my $dir (split(/\//,$file)) {
-				$dircollect.="$dir/";
-				# Use a hash to prevent duplicate chmods.
-				$tochmod{"$workdir/$dircollect"}=1;
-			}
+	# cpio does not necessarily store all parent directories in an
+	# archive, and so some directories, if it has to make them and has
+	# no permission info, may come out mode 700. Here I just chown all
+	# extracted directories to mode 755, which is more reasonable.
+	# Note that the next section overrides these default permissions,
+	# if override data exists in the rpm permissions info. And such
+	# data should always exist, so this is probably a no-op.
+	$this->do("find $workdir -type d -perm 700 -print0 | xargs --no-run-if-empty -0 chmod 700");
+	
+	# rpm files have two sets of permissions; the set in the cpio
+	# archive, and the set in the control data; which override them.
+	# The set in the control data are more correct, so let's use those.
+	# Some permissions setting may have to be postponed until the
+	# postinst.
+	my %owninfo = ();
+	my %modeinfo = ();
+	open (GETPERMS, 'rpm --queryformat \'[%{FILEMODES} %{FILEUSERNAME} %{FILEGROUPNAME} %{FILENAMES}\n]\' -qp '.$this->filename.' |');
+	while (<GETPERMS>) {
+		chomp;
+		my ($mode, $owner, $group, $file) = split(/ /, $_, 4);
+		$mode = $mode & 07777; # remove filetype
+		my $uid = getpwnam($owner);
+		if (! defined $uid || $uid != 0) {
+			$owninfo{$file}=$owner;
+			$uid=0;
 		}
-		$lastdir=$file if -d "./$file";
+		my $gid = getgrnam($group);
+		if (! defined $gid || $gid != 0) {
+			if (exists $owninfo{$file}) {
+				$owninfo{$file}.=":$group";
+			}
+			else {
+				$owninfo{$file}=":$group";
+			}
+			$gid=0;
+		}
+		if (defined($owninfo{$file}) && ($mode & 07000 > 0)) {
+			$modeinfo{$file} = sprintf "%lo", $mode;
+		}
+		next unless -e "$workdir/$file"; # skip broken links
+		if ($> == 0) {
+			$this->do("chown", "$uid:$gid", "$workdir/$file") 
+				|| die "failed chowning $file to $uid\:$gid\: $!";
+		}
+		$this->do("chmod", sprintf("%lo", $mode), "$workdir/$file") 
+			|| die "failed changing mode of $file to $mode\: $!";
 	}
-	chmod 0755, keys %tochmod if %tochmod;
+	$this->owninfo(\%owninfo);
+	$this->modeinfo(\%modeinfo);
 
 	return 1;
 }
@@ -289,7 +310,7 @@ sub prep {
 	print OUT "\%description\n";
 	print OUT $this->description."\n";
 	print OUT "\n";
-	print OUT "(Converted from a ".$this->origformat." package by alien.)\n";
+	print OUT "(Converted from a ".$this->origformat." package by alien version $Alien::Version.)\n";
 	print OUT "\n";
 	print OUT "%files\n";
 	print OUT $filelist if defined $filelist;
@@ -315,19 +336,19 @@ Build a rpm. If RPMBUILDOPT is set in the environement, the options in
 it are passed to rpm on its command line.
 
 An optional parameter, if passed, can be used to specify the program to use
-to build the rpm. It defaults to rpm.
+to build the rpm. It defaults to rpmbuild.
 
 =cut
 
 sub build {
 	my $this=shift;
-	my $buildcmd=shift || 'rpm';
+	my $buildcmd=shift || 'rpmbuild';
 	my $dir=$this->unpacked_tree || die "The package must be unpacked first!";
 	
 	# Ask rpm how it's set up. We want to know what architecture it
 	# will output, and where it will place rpms.
 	my ($rpmarch, $rpmdir);
-	foreach (`rpm --showrc`) {
+	foreach ($this->runpipe("rpm --showrc")) {
 		chomp;
 		if (/^build arch\s+:\s(.*)$/) {
 			$rpmarch=$1;
@@ -369,7 +390,7 @@ sub build {
 
 	$opts.=" $ENV{RPMBUILDOPTS}" if exists $ENV{RPMBUILDOPTS};
 	my $command="cd $dir; $buildcmd -bb $opts ".$this->name."-".$this->version."-".$this->release.".spec";
-	my $log=`$command 2>&1`;
+	my $log=$this->runpipe("$command 2>&1");
 	if ($?) {
 		die "Package build failed. Here's the log of the command ($command):\n", $log;
 	}
